@@ -50,7 +50,6 @@ static void clck_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	GAtResultIter iter;
 	int status_mask, status, class, line;
 
-	dump_response("clck_query_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	status_mask = 0;
@@ -86,20 +85,18 @@ static void at_call_barring_query(struct ofono_call_barring *cb,
 	GAtChat *chat = ofono_call_barring_get_data(cb);
 	struct cb_data *cbd = cb_data_new(callback, data);
 	char buf[64];
-	int len;
 
-	if (!cbd || strlen(lock) != 2)
+	if (cbd == NULL || strlen(lock) != 2)
 		goto error;
 
-	len = sprintf(buf, "AT+CLCK=\"%s\",2", lock);
+	snprintf(buf, sizeof(buf), "AT+CLCK=\"%s\",2", lock);
 
 	if (g_at_chat_send(chat, buf, clck_prefix,
 				clck_query_cb, cbd, g_free) > 0)
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(callback, 0, data);
 }
@@ -110,7 +107,6 @@ static void clck_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_call_barring_set_cb_t callback = cbd->cb;
 	struct ofono_error error;
 
-	dump_response("clck_set_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 	callback(&error, cbd->data);
 }
@@ -125,7 +121,7 @@ static void at_call_barring_set(struct ofono_call_barring *cb, const char *lock,
 	char buf[64];
 	int len;
 
-	if (!cbd || strlen(lock) != 2 || (cls && !passwd))
+	if (cbd == NULL || strlen(lock) != 2 || (cls && passwd == NULL))
 		goto error;
 
 	len = snprintf(buf, sizeof(buf), "AT+CLCK=\"%s\",%i", lock, enable);
@@ -134,8 +130,7 @@ static void at_call_barring_set(struct ofono_call_barring *cb, const char *lock,
 				",\"%s\"", passwd);
 		/* Assume cls == 7 means use defaults */
 		if (cls != 7)
-			len += snprintf(buf + len, sizeof(buf) - len,
-					",%i", cls);
+			snprintf(buf + len, sizeof(buf) - len, ",%i", cls);
 	}
 
 	if (g_at_chat_send(chat, buf, none_prefix,
@@ -143,8 +138,7 @@ static void at_call_barring_set(struct ofono_call_barring *cb, const char *lock,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(callback, data);
 }
@@ -155,7 +149,6 @@ static void cpwd_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_call_barring_set_cb_t callback = cbd->cb;
 	struct ofono_error error;
 
-	dump_response("cpwd_set_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 	callback(&error, cbd->data);
 }
@@ -171,7 +164,7 @@ static void at_call_barring_set_passwd(struct ofono_call_barring *cb,
 	struct cb_data *cbd = cb_data_new(callback, data);
 	char buf[64];
 
-	if (!cbd || strlen(lock) != 2)
+	if (cbd == NULL || strlen(lock) != 2)
 		goto error;
 
 	snprintf(buf, sizeof(buf), "AT+CPWD=\"%s\",\"%s\",\"%s\"",
@@ -182,8 +175,7 @@ static void at_call_barring_set_passwd(struct ofono_call_barring *cb,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(callback, data);
 }
@@ -202,7 +194,7 @@ static int at_call_barring_probe(struct ofono_call_barring *cb,
 {
 	GAtChat *chat = user;
 
-	ofono_call_barring_set_data(cb, chat);
+	ofono_call_barring_set_data(cb, g_at_chat_clone(chat));
 	g_idle_add(at_call_barring_register, cb);
 
 	return 0;
@@ -210,6 +202,10 @@ static int at_call_barring_probe(struct ofono_call_barring *cb,
 
 static void at_call_barring_remove(struct ofono_call_barring *cb)
 {
+	GAtChat *chat = ofono_call_barring_get_data(cb);
+
+	g_at_chat_unref(chat);
+	ofono_call_barring_set_data(cb, NULL);
 }
 
 static struct ofono_call_barring_driver driver = {
