@@ -27,5 +27,8 @@ extern void huawei_voicecall_exit();
 extern void huawei_audio_settings_init();
 extern void huawei_audio_settings_exit();
 
+extern void huawei_radio_settings_init();
+extern void huawei_radio_settings_exit();
+
 extern void huawei_gprs_context_init();
 extern void huawei_gprs_context_exit();
