@@ -37,6 +37,7 @@
 #include "util.h"
 #include "smsutil.h"
 #include "storage.h"
+#include "simutil.h"
 
 #define uninitialized_var(x) x = x
 
@@ -60,6 +61,13 @@ enum message_state {
 struct message {
 	struct ofono_uuid uuid;
 	enum message_state state;
+	struct tx_queue_entry *entry;
+};
+
+struct sms_handler {
+	struct ofono_watchlist_item item;
+	int dst;
+	int src;
 };
 
 struct ofono_sms {
@@ -69,7 +77,7 @@ struct ofono_sms {
 	struct sms_assembly *assembly;
 	guint ref;
 	GQueue *txq;
-	gint tx_source;
+	guint tx_source;
 	struct ofono_message_waiting *mw;
 	unsigned int mw_watch;
 	struct ofono_sim *sim;
@@ -82,6 +90,8 @@ struct ofono_sms {
 	ofono_bool_t use_delivery_reports;
 	struct status_report_assembly *sr_assembly;
 	GHashTable *messages;
+	struct ofono_watchlist *text_handlers;
+	struct ofono_watchlist *datagram_handlers;
 };
 
 struct pending_pdu {
@@ -108,6 +118,11 @@ static gboolean uuid_equal(gconstpointer v1, gconstpointer v2)
 	return memcmp(v1, v2, OFONO_SHA1_UUID_LEN) == 0;
 }
 
+static gboolean port_equal(int received, int expected)
+{
+	return expected == -1 || received == expected;
+}
+
 static guint uuid_hash(gconstpointer v)
 {
 	const struct ofono_uuid *uuid = v;
@@ -170,6 +185,77 @@ static void append_message_properties(struct message *m, DBusMessageIter *dict)
 	ofono_dbus_dict_append(dict, "State", DBUS_TYPE_STRING, &state);
 }
 
+static unsigned int add_sms_handler(struct ofono_watchlist *watchlist,
+					int dst, int src, void *notify,
+					void *data, ofono_destroy_func destroy)
+{
+	struct sms_handler *handler;
+
+	if (notify == NULL)
+		return 0;
+
+	handler = g_try_new0(struct sms_handler, 1);
+	if (handler == NULL)
+		return 0;
+
+	handler->dst = dst;
+	handler->src = src;
+	handler->item.notify = notify;
+	handler->item.notify_data = data;
+	handler->item.destroy = destroy;
+
+	return __ofono_watchlist_add_item(watchlist,
+				(struct ofono_watchlist_item *) handler);
+}
+
+unsigned int __ofono_sms_text_watch_add(struct ofono_sms *sms,
+					ofono_sms_text_notify_cb_t cb,
+					void *data, ofono_destroy_func destroy)
+{
+	if (sms == NULL)
+		return 0;
+
+	DBG("%p", sms);
+
+	return add_sms_handler(sms->text_handlers, -1, -1, cb, data, destroy);
+}
+
+gboolean __ofono_sms_text_watch_remove(struct ofono_sms *sms,
+					unsigned int id)
+{
+	if (sms == NULL)
+		return FALSE;
+
+	DBG("%p", sms);
+
+	return __ofono_watchlist_remove_item(sms->text_handlers, id);
+}
+
+unsigned int __ofono_sms_datagram_watch_add(struct ofono_sms *sms,
+					ofono_sms_datagram_notify_cb_t cb,
+					int dst, int src, void *data,
+					ofono_destroy_func destroy)
+{
+	if (sms == NULL)
+		return 0;
+
+	DBG("%p: dst %d, src %d", sms, dst, src);
+
+	return add_sms_handler(sms->datagram_handlers, dst, src, cb, data,
+				destroy);
+}
+
+gboolean __ofono_sms_datagram_watch_remove(struct ofono_sms *sms,
+					unsigned int id)
+{
+	if (sms == NULL)
+		return FALSE;
+
+	DBG("%p", sms);
+
+	return __ofono_watchlist_remove_item(sms->datagram_handlers, id);
+}
+
 static DBusMessage *message_get_properties(DBusConnection *conn,
 						DBusMessage *msg, void *data)
 {
@@ -179,8 +265,7 @@ static DBusMessage *message_get_properties(DBusConnection *conn,
 	DBusMessageIter dict;
 
 	reply = dbus_message_new_method_return(msg);
-
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -227,14 +312,14 @@ static void message_destroy(gpointer userdata)
 	g_free(m);
 }
 
-static const char *message_build_path(struct ofono_sms *sms,
-						struct message *m)
+const char *__ofono_sms_message_path_from_uuid(struct ofono_sms *sms,
+						const struct ofono_uuid *uuid)
 {
 	static char path[256];
 
 	snprintf(path, sizeof(path), "%s/message_%s",
 			__ofono_atom_get_path(sms->atom),
-			ofono_uuid_to_str(&m->uuid));
+			ofono_uuid_to_str(uuid));
 
 	return path;
 }
@@ -244,10 +329,10 @@ static gboolean message_dbus_register(struct ofono_sms *sms, struct message *m)
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *path;
 
-	if (!m)
+	if (m == NULL)
 		return FALSE;
 
-	path = message_build_path(sms, m);
+	path = __ofono_sms_message_path_from_uuid(sms, &m->uuid);
 
 	if (!g_dbus_register_interface(conn, path, OFONO_MESSAGE_INTERFACE,
 					message_methods, message_signals,
@@ -265,7 +350,7 @@ static gboolean message_dbus_unregister(struct ofono_sms *sms,
 						struct message *m)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
-	const char *path = message_build_path(sms, m);
+	const char *path = __ofono_sms_message_path_from_uuid(sms, &m->uuid);
 
 	return g_dbus_unregister_interface(conn, path,
 						OFONO_MESSAGE_INTERFACE);
@@ -289,7 +374,7 @@ static void emit_message_added(struct ofono_sms *sms, struct message *m)
 
 	dbus_message_iter_init_append(signal, &iter);
 
-	path = message_build_path(sms, m);
+	path = __ofono_sms_message_path_from_uuid(sms, &m->uuid);
 	dbus_message_iter_append_basic(&iter, DBUS_TYPE_OBJECT_PATH, &path);
 
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
@@ -305,7 +390,7 @@ static void emit_message_removed(struct ofono_sms *sms, struct message *m)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *atompath = __ofono_atom_get_path(sms->atom);
-	const char *path = message_build_path(sms, m);
+	const char *path = __ofono_sms_message_path_from_uuid(sms, &m->uuid);
 
 	g_dbus_emit_signal(conn, atompath, OFONO_MESSAGE_MANAGER_INTERFACE,
 				"MessageRemoved", DBUS_TYPE_OBJECT_PATH, &path,
@@ -330,7 +415,7 @@ static void message_set_state(struct ofono_sms *sms,
 		return;
 
 	m->state = new_state;
-	path = message_build_path(sms, m);
+	path = __ofono_sms_message_path_from_uuid(sms, uuid);
 	state = message_state_to_string(m->state);
 
 	ofono_dbus_signal_property_changed(conn, path,
@@ -340,6 +425,8 @@ static void message_set_state(struct ofono_sms *sms,
 
 	if (m->state == MESSAGE_STATE_SENT ||
 			m->state == MESSAGE_STATE_FAILED) {
+		m->entry = NULL;
+
 		g_hash_table_remove(sms->messages, uuid);
 		emit_message_removed(sms, m);
 		message_dbus_unregister(sms, m);
@@ -398,8 +485,7 @@ static DBusMessage *generate_get_properties_reply(struct ofono_sms *sms,
 	const char *bearer;
 
 	reply = dbus_message_new_method_return(msg);
-
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -453,7 +539,7 @@ static DBusMessage *sms_get_properties(DBusConnection *conn,
 	if (sms->pending)
 		return __ofono_error_busy(msg);
 
-	if (!sms->driver->sca_query)
+	if (sms->driver->sca_query == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (sms->flags & MESSAGE_MANAGER_FLAG_CACHED)
@@ -701,18 +787,24 @@ next_q:
 
 	if (entry->flags & OFONO_SMS_SUBMIT_FLAG_RECORD_HISTORY) {
 		enum ofono_history_sms_status hs;
-		enum message_state ms;
 
-		if (ok) {
+		if (ok)
 			hs = OFONO_HISTORY_SMS_STATUS_SUBMITTED;
-			ms = MESSAGE_STATE_SENT;
-		} else {
+		else
 			hs = OFONO_HISTORY_SMS_STATUS_SUBMIT_FAILED;
-			ms = MESSAGE_STATE_FAILED;
-		}
 
 		__ofono_history_sms_send_status(modem, &entry->uuid,
 						time(NULL), hs);
+	}
+
+	if (entry->flags & OFONO_SMS_SUBMIT_FLAG_EXPOSE_DBUS) {
+		enum message_state ms;
+
+		if (ok)
+			ms = MESSAGE_STATE_SENT;
+		else
+			ms = MESSAGE_STATE_FAILED;
+
 		message_set_state(sms, &entry->uuid, ms);
 	}
 
@@ -738,7 +830,7 @@ static gboolean tx_next(gpointer user_data)
 
 	sms->tx_source = 0;
 
-	if (!entry)
+	if (entry == NULL)
 		return FALSE;
 
 	if (g_queue_get_length(sms->txq) > 1
@@ -751,29 +843,6 @@ static gboolean tx_next(gpointer user_data)
 	return FALSE;
 }
 
-static void set_ref_and_to(GSList *msg_list, guint16 ref, int offset,
-				gboolean use_16bit, const char *to)
-{
-	GSList *l;
-	struct sms *sms;
-
-	for (l = msg_list; l; l = l->next) {
-		sms = l->data;
-
-		sms_address_from_string(&sms->submit.daddr, to);
-
-		if (offset == 0)
-			continue;
-
-		if (use_16bit) {
-			sms->submit.ud[offset] = (ref & 0xf0) >> 8;
-			sms->submit.ud[offset+1] = ref & 0x0f;
-		} else {
-			sms->submit.ud[offset] = ref & 0x0f;
-		}
-	}
-}
-
 /**
  * Generate a UUID from an SMS PDU List
  *
@@ -816,16 +885,13 @@ static gboolean sms_uuid_from_pdus(const struct pending_pdu *pdu,
 }
 
 static struct tx_queue_entry *tx_queue_entry_new(GSList *msg_list,
-						unsigned int flags,
-						ofono_sms_txq_submit_cb_t cb,
-						void *data,
-						ofono_destroy_func destroy)
+							unsigned int flags)
 {
 	struct tx_queue_entry *entry;
 	int i = 0;
 	GSList *l;
 
-	entry= g_try_new0(struct tx_queue_entry, 1);
+	entry = g_try_new0(struct tx_queue_entry, 1);
 	if (entry == NULL)
 		return NULL;
 
@@ -843,9 +909,6 @@ static struct tx_queue_entry *tx_queue_entry_new(GSList *msg_list,
 	}
 
 	entry->flags = flags;
-	entry->cb = cb;
-	entry->data = data;
-	entry->destroy = destroy;
 
 	for (l = msg_list; l; l = l->next) {
 		struct pending_pdu *pdu = &entry->pdus[i++];
@@ -867,6 +930,28 @@ error:
 	return NULL;
 }
 
+static void tx_queue_entry_set_submit_notify(struct tx_queue_entry *entry,
+						ofono_sms_txq_submit_cb_t cb,
+						void *data,
+						ofono_destroy_func destroy)
+{
+	entry->cb = cb;
+	entry->data = data;
+	entry->destroy = destroy;
+}
+
+static void message_queued(struct ofono_sms *sms,
+				const struct ofono_uuid *uuid, void *data)
+{
+	DBusConnection *conn = ofono_dbus_get_connection();
+	DBusMessage *msg = data;
+	const char *path;
+
+	path = __ofono_sms_message_path_from_uuid(sms, uuid);
+	g_dbus_send_reply(conn, msg, DBUS_TYPE_OBJECT_PATH, &path,
+					DBUS_TYPE_INVALID);
+}
+
 /*
  * Pre-process a SMS text message and deliver it [D-Bus SendMessage()]
  *
@@ -887,13 +972,11 @@ static DBusMessage *sms_send_message(DBusConnection *conn, DBusMessage *msg,
 	const char *to;
 	const char *text;
 	GSList *msg_list;
-	int ref_offset;
 	struct ofono_modem *modem;
 	unsigned int flags;
 	gboolean use_16bit_ref = FALSE;
-	struct tx_queue_entry *entry;
-	struct message *m;
-	const char *path;
+	int err;
+	struct ofono_uuid uuid;
 
 	if (!dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &to,
 					DBUS_TYPE_STRING, &text,
@@ -903,63 +986,31 @@ static DBusMessage *sms_send_message(DBusConnection *conn, DBusMessage *msg,
 	if (valid_phone_number_format(to) == FALSE)
 		return __ofono_error_invalid_format(msg);
 
-	msg_list = sms_text_prepare(text, 0, use_16bit_ref, &ref_offset,
-					sms->use_delivery_reports);
+	msg_list = sms_text_prepare(to, text, sms->ref, use_16bit_ref,
+						sms->use_delivery_reports);
 
-	if (!msg_list)
+	if (msg_list == NULL)
 		return __ofono_error_invalid_format(msg);
 
-	set_ref_and_to(msg_list, sms->ref, ref_offset, use_16bit_ref, to);
-	DBG("ref: %d, offset: %d", sms->ref, ref_offset);
-
-	if (ref_offset != 0) {
-		if (sms->ref == 65536)
-			sms->ref = 1;
-		else
-			sms->ref = sms->ref + 1;
-	}
-
 	flags = OFONO_SMS_SUBMIT_FLAG_RECORD_HISTORY;
 	flags |= OFONO_SMS_SUBMIT_FLAG_RETRY;
+	flags |= OFONO_SMS_SUBMIT_FLAG_EXPOSE_DBUS;
 	if (sms->use_delivery_reports)
 		flags |= OFONO_SMS_SUBMIT_FLAG_REQUEST_SR;
 
-	entry = tx_queue_entry_new(msg_list, flags, NULL, NULL, NULL);
+	err = __ofono_sms_txq_submit(sms, msg_list, flags, &uuid,
+					message_queued, msg);
 
-	g_slist_foreach(msg_list, (GFunc)g_free, NULL);
+	g_slist_foreach(msg_list, (GFunc) g_free, NULL);
 	g_slist_free(msg_list);
 
-	if (entry == NULL)
-		goto err;
-
-	m = message_create(&entry->uuid);
-	if (m == NULL)
-		goto err;
-
-	if (message_dbus_register(sms, m) == FALSE)
-		goto err;
-
-	g_hash_table_insert(sms->messages, &m->uuid, m);
-
-	g_queue_push_tail(sms->txq, entry);
-
-	if (g_queue_get_length(sms->txq) == 1)
-		sms->tx_source = g_timeout_add(0, tx_next, sms);
-
-	path = message_build_path(sms, m);
-	g_dbus_send_reply(conn, msg, DBUS_TYPE_OBJECT_PATH, &path,
-					DBUS_TYPE_INVALID);
+	if (err < 0)
+		return __ofono_error_failed(msg);
 
 	modem = __ofono_atom_get_modem(sms->atom);
-	__ofono_history_sms_send_pending(modem, &entry->uuid,
-						to, time(NULL), text);
-
-	emit_message_added(sms, m);
+	__ofono_history_sms_send_pending(modem, &uuid, to, time(NULL), text);
 
 	return NULL;
-
-err:
-	return __ofono_error_failed(msg);
 }
 
 static DBusMessage *sms_get_messages(DBusConnection *conn, DBusMessage *msg,
@@ -997,7 +1048,7 @@ static DBusMessage *sms_get_messages(DBusConnection *conn, DBusMessage *msg,
 	while (g_hash_table_iter_next(&hashiter, &key, &value)) {
 		m = value;
 
-		path = message_build_path(sms, m);
+		path = __ofono_sms_message_path_from_uuid(sms, &m->uuid);
 
 		dbus_message_iter_open_container(&array, DBUS_TYPE_STRUCT,
 							NULL, &entry);
@@ -1054,7 +1105,11 @@ static gboolean compute_incoming_msgid(GSList *sms_list,
 	for (l = sms_list; l; l = l->next) {
 		s = l->data;
 
-		sms_encode(s, &len, NULL, buf);
+		if (sms_encode(s, &len, NULL, buf) == FALSE) {
+			g_checksum_free(checksum);
+			return FALSE;
+		}
+
 		g_checksum_update(checksum, buf, len);
 	}
 
@@ -1064,12 +1119,35 @@ static gboolean compute_incoming_msgid(GSList *sms_list,
 	return TRUE;
 }
 
-static void dispatch_app_datagram(struct ofono_sms *sms, int dst, int src,
-					unsigned char *buf, long len)
+static void dispatch_app_datagram(struct ofono_sms *sms,
+					const struct ofono_uuid *uuid,
+					int dst, int src,
+					unsigned char *buf, unsigned len,
+					const struct sms_address *addr,
+					const struct sms_scts *scts)
 {
-	DBG("Got app datagram for dst port: %d, src port: %d",
-			dst, src);
-	DBG("Contents-Len: %ld", len);
+	const char *sender = sms_address_to_string(addr);
+	time_t ts;
+	struct tm remote;
+	struct tm local;
+
+	ofono_sms_datagram_notify_cb_t notify;
+	struct sms_handler *h;
+	GSList *l;
+
+	ts = sms_scts_to_time(scts, &remote);
+	localtime_r(&ts, &local);
+
+	for (l = sms->datagram_handlers->items; l; l = l->next) {
+		h = l->data;
+		notify = h->item.notify;
+
+		if (!port_equal(dst, h->dst) || !port_equal(src, h->src))
+			continue;
+
+		notify(sender, &remote, &local, dst, src, buf, len,
+			h->item.notify_data);
+	}
 }
 
 static void dispatch_text_message(struct ofono_sms *sms,
@@ -1091,8 +1169,11 @@ static void dispatch_text_message(struct ofono_sms *sms,
 	struct tm remote;
 	struct tm local;
 	const char *str = buf;
+	ofono_sms_text_notify_cb_t notify;
+	struct sms_handler *h;
+	GSList *l;
 
-	if (!message)
+	if (message == NULL)
 		return;
 
 	if (cls == SMS_CLASS_0)
@@ -1103,7 +1184,7 @@ static void dispatch_text_message(struct ofono_sms *sms,
 	signal = dbus_message_new_signal(path, OFONO_MESSAGE_MANAGER_INTERFACE,
 						signal_name);
 
-	if (!signal)
+	if (signal == NULL)
 		return;
 
 	dbus_message_iter_init_append(signal, &iter);
@@ -1132,15 +1213,25 @@ static void dispatch_text_message(struct ofono_sms *sms,
 
 	g_dbus_send_message(conn, signal);
 
-	if (cls != SMS_CLASS_0)
-		__ofono_history_sms_received(modem, uuid, str,
-						&remote, &local, message);
+	if (cls == SMS_CLASS_0)
+		return;
+
+	for (l = sms->text_handlers->items; l; l = l->next) {
+		h = l->data;
+		notify = h->item.notify;
+
+		notify(str, &remote, &local, message, h->item.notify_data);
+	}
+
+	__ofono_history_sms_received(modem, uuid, str, &remote, &local,
+					message);
 }
 
 static void sms_dispatch(struct ofono_sms *sms, GSList *sms_list)
 {
 	GSList *l;
 	const struct sms *s;
+	struct ofono_uuid uuid;
 	enum sms_charset uninitialized_var(old_charset);
 	enum sms_class cls;
 	int srcport = -1;
@@ -1197,8 +1288,8 @@ static void sms_dispatch(struct ofono_sms *sms, GSList *sms_list)
 		}
 
 		if (sms_extract_app_port(s, &cdst, &csrc, &is_8bit)) {
-			csrc = is_8bit ? (csrc << 8) : csrc;
-			cdst = is_8bit ? (cdst << 8) : cdst;
+			csrc = is_8bit ? (csrc << 16) : csrc;
+			cdst = is_8bit ? (cdst << 16) : cdst;
 
 			if (l == sms_list) {
 				srcport = csrc;
@@ -1206,6 +1297,8 @@ static void sms_dispatch(struct ofono_sms *sms, GSList *sms_list)
 			}
 		}
 
+		DBG("dst %d src %d 8bit %u", cdst, csrc, is_8bit);
+
 		if (srcport != csrc || dstport != cdst) {
 			ofono_error("Source / Destination ports across "
 					"concatenated message are not the "
@@ -1214,6 +1307,11 @@ static void sms_dispatch(struct ofono_sms *sms, GSList *sms_list)
 		}
 	}
 
+	if (!compute_incoming_msgid(sms_list, &uuid))
+		return;
+
+	s = sms_list->data;
+
 	/* Handle datagram */
 	if (old_charset == SMS_CHARSET_8BIT) {
 		unsigned char *buf;
@@ -1226,27 +1324,21 @@ static void sms_dispatch(struct ofono_sms *sms, GSList *sms_list)
 		}
 
 		buf = sms_decode_datagram(sms_list, &len);
-
-		if (!buf)
+		if (buf == NULL)
 			return;
 
-		dispatch_app_datagram(sms, dstport, srcport, buf, len);
+		dispatch_app_datagram(sms, &uuid, dstport, srcport, buf, len,
+					&s->deliver.oaddr, &s->deliver.scts);
 
 		g_free(buf);
 	} else {
-		struct ofono_uuid uuid;
 		char *message = sms_decode_text(sms_list);
 
-		if (!message)
+		if (message == NULL)
 			return;
 
-		if (compute_incoming_msgid(sms_list, &uuid)) {
-			s = sms_list->data;
-
-			dispatch_text_message(sms, &uuid, message, cls,
-						&s->deliver.oaddr,
-						&s->deliver.scts);
-		}
+		dispatch_text_message(sms, &uuid, message, cls,
+					&s->deliver.oaddr, &s->deliver.scts);
 
 		g_free(message);
 	}
@@ -1264,25 +1356,25 @@ static void handle_deliver(struct ofono_sms *sms, const struct sms *incoming)
 	if (sms_extract_concatenation(incoming, &ref, &max, &seq)) {
 		GSList *sms_list;
 
-		if (!sms->assembly)
+		if (sms->assembly == NULL)
 			return;
 
 		sms_list = sms_assembly_add_fragment(sms->assembly,
-							incoming, time(NULL),
-							&incoming->deliver.oaddr,
-							ref, max, seq);
+						incoming, time(NULL),
+						&incoming->deliver.oaddr,
+						ref, max, seq);
 
-		if (!sms_list)
+		if (sms_list == NULL)
 			return;
 
 		sms_dispatch(sms, sms_list);
-		g_slist_foreach(sms_list, (GFunc)g_free, NULL);
+		g_slist_foreach(sms_list, (GFunc) g_free, NULL);
 		g_slist_free(sms_list);
 
 		return;
 	}
 
-	l = g_slist_append(NULL, (void *)incoming);
+	l = g_slist_append(NULL, (void *) incoming);
 	sms_dispatch(sms, l);
 	g_slist_free(l);
 }
@@ -1323,6 +1415,9 @@ static inline gboolean handle_mwi(struct ofono_sms *sms, struct sms *s)
 void ofono_sms_deliver_notify(struct ofono_sms *sms, unsigned char *pdu,
 				int len, int tpdu_len)
 {
+	struct ofono_modem *modem = __ofono_atom_get_modem(sms->atom);
+	struct ofono_atom *stk_atom;
+	struct ofono_atom *sim_atom;
 	struct sms s;
 	enum sms_class cls;
 
@@ -1387,13 +1482,36 @@ void ofono_sms_deliver_notify(struct ofono_sms *sms, unsigned char *pdu,
 		break;
 	case SMS_PID_TYPE_USIM_DOWNLOAD:
 	case SMS_PID_TYPE_ANSI136:
-		if (cls == SMS_CLASS_2) {
-			ofono_error("(U)SIM Download messages not supported");
-			return;
-		}
+		/* If not Class 2, handle in a "normal" way */
+		if (cls != SMS_CLASS_2)
+			break;
 
-		/* Otherwise handle in a "normal" way */
-		break;
+		sim_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_SIM);
+
+		if (sim_atom == NULL)
+			return;
+
+		if (!__ofono_sim_service_available(
+					__ofono_atom_get_data(sim_atom),
+					SIM_UST_SERVICE_DATA_DOWNLOAD_SMS_PP,
+					SIM_SST_SERVICE_DATA_DOWNLOAD_SMS_PP))
+			return;
+
+		stk_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_STK);
+
+		if (stk_atom == NULL)
+			return;
+
+		__ofono_sms_sim_download(__ofono_atom_get_data(stk_atom),
+						&s, NULL, sms);
+
+		/*
+		 * Passing the USIM response back to network is not
+		 * currently supported
+		 *
+		 * TODO: store in EFsms if not handled
+		 */
+		return;
 	default:
 		break;
 	}
@@ -1468,7 +1586,7 @@ void ofono_sms_status_notify(struct ofono_sms *sms, unsigned char *pdu,
 		return;
 	}
 
-	if (!sms_dcs_decode(s.deliver.dcs, &cls, NULL, NULL, NULL)) {
+	if (!sms_dcs_decode(s.status_report.dcs, &cls, NULL, NULL, NULL)) {
 		ofono_error("Unknown / Reserved DCS.  Ignoring");
 		return;
 	}
@@ -1483,7 +1601,7 @@ int ofono_sms_driver_register(const struct ofono_sms_driver *d)
 	if (d->probe == NULL)
 		return -EINVAL;
 
-	g_drivers = g_slist_prepend(g_drivers, (void *)d);
+	g_drivers = g_slist_prepend(g_drivers, (void *) d);
 
 	return 0;
 }
@@ -1492,7 +1610,7 @@ void ofono_sms_driver_unregister(const struct ofono_sms_driver *d)
 {
 	DBG("driver: %p, name: %s", d, d->name);
 
-	g_drivers = g_slist_remove(g_drivers, (void *)d);
+	g_drivers = g_slist_remove(g_drivers, (void *) d);
 }
 
 static void sms_unregister(struct ofono_atom *atom)
@@ -1527,6 +1645,12 @@ static void sms_unregister(struct ofono_atom *atom)
 		g_hash_table_destroy(sms->messages);
 		sms->messages = NULL;
 	}
+
+	__ofono_watchlist_free(sms->text_handlers);
+	sms->text_handlers = NULL;
+
+	__ofono_watchlist_free(sms->datagram_handlers);
+	sms->datagram_handlers = NULL;
 }
 
 static void sms_remove(struct ofono_atom *atom)
@@ -1741,6 +1865,9 @@ void ofono_sms_register(struct ofono_sms *sms)
 		sms->driver->bearer_set(sms, sms->bearer,
 						bearer_init_callback, sms);
 
+	sms->text_handlers = __ofono_watchlist_new(g_free);
+	sms->datagram_handlers = __ofono_watchlist_new(g_free);
+
 	__ofono_atom_register(sms->atom, sms_unregister);
 }
 
@@ -1759,18 +1886,42 @@ void *ofono_sms_get_data(struct ofono_sms *sms)
 	return sms->driver_data;
 }
 
+unsigned short __ofono_sms_get_next_ref(struct ofono_sms *sms)
+{
+	return sms->ref;
+}
+
 int __ofono_sms_txq_submit(struct ofono_sms *sms, GSList *list,
 				unsigned int flags,
 				struct ofono_uuid *uuid,
-				ofono_sms_txq_submit_cb_t cb,
-				void *data, ofono_destroy_func destroy)
+				ofono_sms_txq_queued_cb_t cb, void *data)
 {
+	struct message *m = NULL;
 	struct tx_queue_entry *entry;
 
-	entry = tx_queue_entry_new(list, flags, cb, data, destroy);
+	entry = tx_queue_entry_new(list, flags);
 	if (entry == NULL)
 		return -ENOMEM;
 
+	if (flags & OFONO_SMS_SUBMIT_FLAG_EXPOSE_DBUS) {
+		m = message_create(&entry->uuid);
+		if (m == NULL)
+			goto err;
+
+		if (message_dbus_register(sms, m) == FALSE)
+			goto err;
+
+		g_hash_table_insert(sms->messages, &m->uuid, m);
+		m->entry = entry;
+	}
+
+	if (list->next != NULL) {
+		if (sms->ref == 65536)
+			sms->ref = 1;
+		else
+			sms->ref = sms->ref + 1;
+	}
+
 	g_queue_push_tail(sms->txq, entry);
 
 	if (g_queue_get_length(sms->txq) == 1)
@@ -1779,20 +1930,36 @@ int __ofono_sms_txq_submit(struct ofono_sms *sms, GSList *list,
 	if (uuid)
 		memcpy(uuid, &entry->uuid, sizeof(*uuid));
 
-	if (flags & OFONO_SMS_SUBMIT_FLAG_RECORD_HISTORY) {
-		struct message *m;
+	if (cb)
+		cb(sms, &entry->uuid, data);
 
-		m = message_create(&entry->uuid);
-		if (m == NULL)
-			goto out;
-
-		if (message_dbus_register(sms, m) == FALSE)
-			goto out;
-
-		g_hash_table_insert(sms->messages, &m->uuid, m);
+	if (m && (flags & OFONO_SMS_SUBMIT_FLAG_EXPOSE_DBUS))
 		emit_message_added(sms, m);
-	}
 
-out:
+	return 0;
+
+err:
+	tx_queue_entry_destroy(entry);
+
+	return -EINVAL;
+}
+
+int __ofono_sms_txq_set_submit_notify(struct ofono_sms *sms,
+					struct ofono_uuid *uuid,
+					ofono_sms_txq_submit_cb_t cb,
+					void *data,
+					ofono_destroy_func destroy)
+{
+	struct message *m;
+
+	m = g_hash_table_lookup(sms->messages, uuid);
+	if (m == NULL)
+		return -ENOENT;
+
+	if (m->entry == NULL)
+		return -ENOTSUP;
+
+	tx_queue_entry_set_submit_notify(m->entry, cb, data, destroy);
+
 	return 0;
 }
