@@ -45,7 +45,18 @@
 #define SMS_BACKUP_PATH_DIR SMS_BACKUP_PATH "/%s-%i-%i"
 #define SMS_BACKUP_PATH_FILE SMS_BACKUP_PATH_DIR "/%03i"
 
+#define SMS_SR_BACKUP_PATH STORAGEDIR "/%s/sms_sr"
+#define SMS_SR_BACKUP_PATH_FILE SMS_SR_BACKUP_PATH "/%s-%s"
+
 #define SMS_ADDR_FMT "%24[0-9A-F]"
+#define SMS_MSGID_FMT "%40[0-9A-F]"
+
+/*
+ * Time zone accounts for daylight saving time, and the two extreme time
+ * zones on earth are UTC-12 and UTC+14.
+ */
+#define MAX_TIMEZONE 56
+#define MIN_TIMEZONE -48
 
 static GSList *sms_assembly_add_fragment_backup(struct sms_assembly *assembly,
 					const struct sms *sms, time_t ts,
@@ -154,7 +165,7 @@ gboolean sms_dcs_decode(guint8 dcs, enum sms_class *cls,
 		comp = (dcs & 0x20) ? TRUE : FALSE;
 
 		if (dcs & 0x10)
-			cl = (enum sms_class)(dcs & 0x03);
+			cl = (enum sms_class) (dcs & 0x03);
 		else
 			cl = SMS_CLASS_UNSPECIFIED;
 
@@ -173,7 +184,7 @@ gboolean sms_dcs_decode(guint8 dcs, enum sms_class *cls,
 		else
 			ch = SMS_CHARSET_7BIT;
 
-		cl = (enum sms_class)(dcs & 0x03);
+		cl = (enum sms_class) (dcs & 0x03);
 
 		break;
 	default:
@@ -215,7 +226,12 @@ gboolean sms_mwi_dcs_decode(guint8 dcs, enum sms_mwi_type *type,
 	else
 		dis = FALSE;
 
-	if (upper == 3)
+	/*
+	 * As per 3GPP TS 23.038 specification, if bits 7..4 set to 1110,
+	 * text included in the user data is coded in the uncompresssed
+	 * UCS2 character set.
+	 */
+	if (upper == 2)
 		ch = SMS_CHARSET_UCS2;
 	else
 		ch = SMS_CHARSET_7BIT;
@@ -335,7 +351,8 @@ gboolean sms_encode_scts(const struct sms_scts *in, unsigned char *pdu,
 	if (in->second > 59)
 		return FALSE;
 
-	if ((in->timezone > 12*4-1) || (in->timezone < -(12*4-1)))
+	if ((in->timezone > MAX_TIMEZONE || in->timezone < MIN_TIMEZONE) &&
+			in->has_timezone == TRUE)
 		return FALSE;
 
 	pdu = pdu + *offset;
@@ -347,6 +364,11 @@ gboolean sms_encode_scts(const struct sms_scts *in, unsigned char *pdu,
 	pdu[4] = ((in->minute / 10) & 0x0f) | (((in->minute % 10) & 0x0f) << 4);
 	pdu[5] = ((in->second / 10) & 0x0f) | (((in->second % 10) & 0x0f) << 4);
 
+	if (in->has_timezone == FALSE) {
+		pdu[6] = 0xff;
+		goto out;
+	}
+
 	timezone = abs(in->timezone);
 
 	pdu[6] = ((timezone / 10) & 0x07) | (((timezone % 10) & 0x0f) << 4);
@@ -354,6 +376,7 @@ gboolean sms_encode_scts(const struct sms_scts *in, unsigned char *pdu,
 	if (in->timezone < 0)
 		pdu[6] |= 0x8;
 
+out:
 	*offset += 7;
 
 	return TRUE;
@@ -375,21 +398,39 @@ gboolean sms_decode_scts(const unsigned char *pdu, int len,
 	next_octet(pdu, len, offset, &oct);
 	out->year = sms_decode_semi_octet(oct);
 
+	if (out->year > 99)
+		return FALSE;
+
 	next_octet(pdu, len, offset, &oct);
 	out->month = sms_decode_semi_octet(oct);
 
+	if (out->month > 12)
+		return FALSE;
+
 	next_octet(pdu, len, offset, &oct);
 	out->day = sms_decode_semi_octet(oct);
 
+	if (out->day > 31)
+		return FALSE;
+
 	next_octet(pdu, len, offset, &oct);
 	out->hour = sms_decode_semi_octet(oct);
 
+	if (out->hour > 23)
+		return FALSE;
+
 	next_octet(pdu, len, offset, &oct);
 	out->minute = sms_decode_semi_octet(oct);
 
+	if (out->minute > 59)
+		return FALSE;
+
 	next_octet(pdu, len, offset, &oct);
 	out->second = sms_decode_semi_octet(oct);
 
+	if (out->second > 59)
+		return FALSE;
+
 	next_octet(pdu, len, offset, &oct);
 
 	/*
@@ -404,6 +445,11 @@ gboolean sms_decode_scts(const unsigned char *pdu, int len,
 	if (oct & 0x08)
 		out->timezone = out->timezone * -1;
 
+	if ((out->timezone > MAX_TIMEZONE) || (out->timezone < MIN_TIMEZONE))
+		return FALSE;
+
+	out->has_timezone = TRUE;
+
 	return TRUE;
 }
 
@@ -500,8 +546,7 @@ gboolean sms_encode_address_field(const struct sms_address *in, gboolean sc,
 			return FALSE;
 
 		gsm = convert_utf8_to_gsm(in->address, len, NULL, &written, 0);
-
-		if (!gsm)
+		if (gsm == NULL)
 			return FALSE;
 
 		r = pack_7bit_own_buf(gsm, written, 0, FALSE, &packed, 0, p);
@@ -616,14 +661,14 @@ gboolean sms_decode_address_field(const unsigned char *pdu, int len,
 
 		*offset = *offset + (addr_len + 1) / 2;
 
-		if (!res)
+		if (res == NULL)
 			return FALSE;
 
 		utf8 = convert_gsm_to_utf8(res, written, NULL, NULL, 0);
 
 		g_free(res);
 
-		if (!utf8)
+		if (utf8 == NULL)
 			return FALSE;
 
 		if (strlen(utf8) > 20) {
@@ -1000,8 +1045,6 @@ static gboolean decode_status_report(const unsigned char *pdu, int len,
 	if (out->status_report.pi & 0x02) {
 		if (!next_octet(pdu, len, &offset, &out->status_report.dcs))
 			return FALSE;
-	} else {
-		out->status_report.dcs = 0;
 	}
 
 	if (out->status_report.pi & 0x04) {
@@ -1479,12 +1522,14 @@ gboolean sms_decode(const unsigned char *pdu, int len, gboolean outgoing,
 	unsigned char type;
 	int offset = 0;
 
-	if (!out)
+	if (out == NULL)
 		return FALSE;
 
 	if (len == 0)
 		return FALSE;
 
+	memset(out, 0, sizeof(*out));
+
 	if (tpdu_len < len) {
 		if (!sms_decode_address_field(pdu, len, &offset,
 						TRUE, &out->sc_addr))
@@ -1592,7 +1637,7 @@ const guint8 *sms_extract_common(const struct sms *sms, gboolean *out_udhi,
 		break;
 	};
 
-	if (!ud)
+	if (ud == NULL)
 		return NULL;
 
 	if (out_udhi)
@@ -1657,8 +1702,7 @@ gboolean sms_udh_iter_init(const struct sms *sms, struct sms_udh_iter *iter)
 	guint8 max_ud_len;
 
 	hdr = sms_extract_common(sms, &udhi, &dcs, &udl, &max_ud_len);
-
-	if (!hdr)
+	if (hdr == NULL)
 		return FALSE;
 
 	if (!udhi)
@@ -1857,7 +1901,7 @@ static gboolean extract_app_port_common(struct sms_udh_iter *iter, int *dst,
 	 * processing at the next information element) any information element
 	 * where the value of the Information-Element-Data is Reserved or not
 	 * supported.
-	*/
+	 */
 	while ((iei = sms_udh_iter_get_ie_type(iter)) !=
 			SMS_IEI_INVALID) {
 		switch (iei) {
@@ -2097,8 +2141,7 @@ unsigned char *sms_decode_datagram(GSList *sms_list, long *out_len)
 		sms = l->data;
 
 		ud = sms_extract_common(sms, NULL, NULL, &udl, NULL);
-
-		if (!ud)
+		if (ud == NULL)
 			return NULL;
 
 		/*
@@ -2116,8 +2159,7 @@ unsigned char *sms_decode_datagram(GSList *sms_list, long *out_len)
 		return NULL;
 
 	buf = g_try_new(unsigned char, len);
-
-	if (!buf)
+	if (buf == NULL)
 		return NULL;
 
 	len = 0;
@@ -2218,7 +2260,8 @@ char *sms_decode_text(GSList *sms_list)
 			if (buf[written-1] == 0x1b)
 				written = written - 1;
 
-			sms_extract_language_variant(sms, &locking_shift, &single_shift);
+			sms_extract_language_variant(sms, &locking_shift,
+								&single_shift);
 
 			/*
 			 * If language is not defined in 3GPP TS 23.038,
@@ -2235,7 +2278,7 @@ char *sms_decode_text(GSList *sms_list)
 								locking_shift,
 								single_shift);
 		} else {
-			const gchar *from = (const gchar *)(ud + taken);
+			const gchar *from = (const gchar *) (ud + taken);
 			/*
 			 * According to the spec: A UCS2 character shall not be
 			 * split in the middle; if the length of the User Data
@@ -2293,8 +2336,7 @@ static gboolean sms_assembly_extract_address(const char *straddr,
 	return sms_decode_address_field(pdu, len, &offset, FALSE, out);
 }
 
-static gboolean sms_assembly_encode_address(const struct sms_address *in,
-						char *straddr)
+gboolean sms_address_to_hex_string(const struct sms_address *in, char *straddr)
 {
 	unsigned char pdu[12];
 	int offset = 0;
@@ -2314,7 +2356,7 @@ static void sms_assembly_load(struct sms_assembly *assembly,
 				const struct dirent *dir)
 {
 	struct sms_address addr;
-	char straddr[25];
+	DECLARE_SMS_ADDR_STR(straddr);
 	guint16 ref;
 	guint8 max;
 	guint8 seq;
@@ -2391,12 +2433,12 @@ static gboolean sms_assembly_store(struct sms_assembly *assembly,
 {
 	unsigned char buf[177];
 	int len;
-	char straddr[25];
+	DECLARE_SMS_ADDR_STR(straddr);
 
-	if (!assembly->imsi)
+	if (assembly->imsi == NULL)
 		return FALSE;
 
-	if (sms_assembly_encode_address(&node->addr, straddr) == FALSE)
+	if (sms_address_to_hex_string(&node->addr, straddr) == FALSE)
 		return FALSE;
 
 	len = sms_serialize(buf, sms);
@@ -2414,12 +2456,12 @@ static void sms_assembly_backup_free(struct sms_assembly *assembly,
 {
 	char *path;
 	int seq;
-	char straddr[25];
+	DECLARE_SMS_ADDR_STR(straddr);
 
-	if (!assembly->imsi)
+	if (assembly->imsi == NULL)
 		return;
 
-	if (sms_assembly_encode_address(&node->addr, straddr) == FALSE)
+	if (sms_address_to_hex_string(&node->addr, straddr) == FALSE)
 		return;
 
 	for (seq = 0; seq < node->max_fragments; seq++) {
@@ -2478,7 +2520,7 @@ void sms_assembly_free(struct sms_assembly *assembly)
 	for (l = assembly->assembly_list; l; l = l->next) {
 		struct sms_assembly_node *node = l->data;
 
-		g_slist_foreach(node->fragment_list, (GFunc)g_free, 0);
+		g_slist_foreach(node->fragment_list, (GFunc) g_free, 0);
 		g_slist_free(node->fragment_list);
 		g_free(node);
 	}
@@ -2530,7 +2572,8 @@ static GSList *sms_assembly_add_fragment_backup(struct sms_assembly *assembly,
 		if (ref != node->ref)
 			continue;
 
-		/* Message Reference and address the same, but max is not
+		/*
+		 * Message Reference and address the same, but max is not
 		 * ignore the SMS completely
 		 */
 		if (max != node->max_fragments)
@@ -2628,7 +2671,7 @@ void sms_assembly_expire(struct sms_assembly *assembly, time_t before)
 
 		sms_assembly_backup_free(assembly, node);
 
-		g_slist_foreach(node->fragment_list, (GFunc)g_free, 0);
+		g_slist_foreach(node->fragment_list, (GFunc) g_free, 0);
 		g_slist_free(node->fragment_list);
 		g_free(node);
 
@@ -2643,20 +2686,182 @@ void sms_assembly_expire(struct sms_assembly *assembly, time_t before)
 	}
 }
 
+static gboolean sha1_equal(gconstpointer v1, gconstpointer v2)
+{
+	return memcmp(v1, v2, SMS_MSGID_LEN) == 0;
+}
+
+static guint sha1_hash(gconstpointer v)
+{
+	guint h;
+
+	memcpy(&h, v, sizeof(h));
+
+	return h;
+}
+
+static void sr_assembly_load_backup(GHashTable *assembly_table,
+					const char *imsi,
+					const struct dirent *addr_dir)
+{
+	struct sms_address addr;
+	DECLARE_SMS_ADDR_STR(straddr);
+	struct id_table_node *node;
+	GHashTable *id_table;
+	int r;
+	char *assembly_table_key;
+	unsigned int *id_table_key;
+	char msgid_str[SMS_MSGID_LEN * 2 + 1];
+	unsigned char msgid[SMS_MSGID_LEN];
+
+	if (addr_dir->d_type != DT_REG)
+		return;
+
+	/*
+	 * All SMS-messages under the same IMSI-code are
+	 * included in the same directory.
+	 * So, SMS-address and message ID are included in the same file name
+	 * Max of SMS address size is 12 bytes, hex encoded
+	 * Max of SMS SHA1 hash is 20 bytes, hex encoded
+	 */
+	if (sscanf(addr_dir->d_name, SMS_ADDR_FMT "-" SMS_MSGID_FMT,
+				straddr, msgid_str) < 2)
+		return;
+
+	if (sms_assembly_extract_address(straddr, &addr) == FALSE)
+		return;
+
+	if (strlen(msgid_str) != 2 * SMS_MSGID_LEN)
+		return;
+
+	if (decode_hex_own_buf(msgid_str, 2 * SMS_MSGID_LEN,
+				NULL, 0, msgid) == NULL)
+		return;
+
+	node = g_new0(struct id_table_node, 1);
+
+	r = read_file((unsigned char *) node,
+			sizeof(struct id_table_node),
+			SMS_SR_BACKUP_PATH "/%s",
+			imsi, addr_dir->d_name);
+
+	if (r < 0) {
+		g_free(node);
+		return;
+	}
+
+	id_table = g_hash_table_lookup(assembly_table,
+					sms_address_to_string(&addr));
+
+	/* Create hashtable keyed by the to address if required */
+	if (id_table == NULL) {
+		id_table = g_hash_table_new_full(sha1_hash, sha1_equal,
+							g_free, g_free);
+
+		assembly_table_key = g_strdup(sms_address_to_string(&addr));
+		g_hash_table_insert(assembly_table, assembly_table_key,
+					id_table);
+	}
+
+	/* Node ready, create key and add them to the table */
+	id_table_key = g_memdup(msgid, SMS_MSGID_LEN);
+
+	g_hash_table_insert(id_table, id_table_key, node);
+}
+
 struct status_report_assembly *status_report_assembly_new(const char *imsi)
 {
+	char *path;
+	int len;
+	struct dirent **addresses;
 	struct status_report_assembly *ret =
 				g_new0(struct status_report_assembly, 1);
 
 	ret->assembly_table = g_hash_table_new_full(g_str_hash, g_str_equal,
-				g_free, (GDestroyNotify)g_hash_table_destroy);
+				g_free, (GDestroyNotify) g_hash_table_destroy);
 
-	if (imsi)
+	if (imsi) {
 		ret->imsi = imsi;
 
+		/* Restore state from backup */
+		path = g_strdup_printf(SMS_SR_BACKUP_PATH, imsi);
+		len = scandir(path, &addresses, NULL, alphasort);
+
+		g_free(path);
+
+		if (len < 0)
+			return ret;
+
+		/*
+		 * Go through different addresses. Each address can relate to
+		 * 1-n msg_ids.
+		 */
+
+		while (len--) {
+			sr_assembly_load_backup(ret->assembly_table, imsi,
+								addresses[len]);
+			g_free(addresses[len]);
+		}
+
+		g_free(addresses);
+	}
+
 	return ret;
 }
 
+static gboolean sr_assembly_add_fragment_backup(const char *imsi,
+					const struct id_table_node *node,
+					const struct sms_address *addr,
+					const unsigned char *msgid)
+{
+	int len = sizeof(struct id_table_node);
+	DECLARE_SMS_ADDR_STR(straddr);
+	char msgid_str[SMS_MSGID_LEN * 2 + 1];
+
+	if (imsi == NULL)
+		return FALSE;
+
+	if (sms_address_to_hex_string(addr, straddr) == FALSE)
+		return FALSE;
+
+	if (encode_hex_own_buf(msgid, SMS_MSGID_LEN, 0, msgid_str) == NULL)
+		return FALSE;
+
+	/* storagedir/%s/sms_sr/%s-%s */
+	if (write_file((unsigned char *) node, len, SMS_BACKUP_MODE,
+			SMS_SR_BACKUP_PATH_FILE, imsi,
+			straddr, msgid_str) != len)
+		return FALSE;
+
+	return TRUE;
+}
+
+static gboolean sr_assembly_remove_fragment_backup(const char *imsi,
+					const struct sms_address *addr,
+					const unsigned char *sha1)
+{
+	char *path;
+	DECLARE_SMS_ADDR_STR(straddr);
+	char msgid_str[SMS_MSGID_LEN * 2 + 1];
+
+	if (imsi == NULL)
+		return FALSE;
+
+	if (sms_address_to_hex_string(addr, straddr) == FALSE)
+		return FALSE;
+
+	if (encode_hex_own_buf(sha1, SMS_MSGID_LEN, 0, msgid_str) == FALSE)
+		return FALSE;
+
+	path = g_strdup_printf(SMS_SR_BACKUP_PATH_FILE,
+					imsi, straddr, msgid_str);
+
+	unlink(path);
+	g_free(path);
+
+	return TRUE;
+}
+
 void status_report_assembly_free(struct status_report_assembly *assembly)
 {
 	g_hash_table_destroy(assembly->assembly_table);
@@ -2684,51 +2889,123 @@ static gboolean sr_st_to_delivered(enum sms_st st, gboolean *delivered)
 	return FALSE;
 }
 
+static struct id_table_node *find_by_mr_and_mark(GHashTable *id_table,
+						unsigned char mr,
+						GHashTableIter *out_iter,
+						unsigned char **out_id)
+{
+	unsigned int offset = mr / 32;
+	unsigned int bit = 1 << (mr % 32);
+	gpointer key, value;
+	struct id_table_node *node;
+
+	g_hash_table_iter_init(out_iter, id_table);
+	while (g_hash_table_iter_next(out_iter, &key, &value)) {
+		node = value;
+
+		/* Address and MR matched */
+		if (node->mrs[offset] & bit) {
+			node->mrs[offset] ^= bit;
+			*out_id = key;
+
+			return node;
+		}
+	}
+
+	return NULL;
+}
+
+/*
+ * Key (receiver address) does not exist in assembly. Some networks can change
+ * address to international format, although address is sent in the national
+ * format. Handle also change from national to international format.
+ * Notify these special cases by comparing only last six digits of the assembly
+ * addresses and received address. If address contains less than six digits,
+ * compare only existing digits.
+ */
+static struct id_table_node *fuzzy_lookup(struct status_report_assembly *assy,
+						const struct sms *sr,
+						const char **out_addr,
+						GHashTableIter *out_iter,
+						unsigned char **out_msgid)
+{
+	GHashTableIter iter_addr;
+	gpointer key, value;
+	const char *r_addr;
+
+	r_addr = sms_address_to_string(&sr->status_report.raddr);
+	g_hash_table_iter_init(&iter_addr, assy->assembly_table);
+
+	while (g_hash_table_iter_next(&iter_addr, &key, &value)) {
+		const char *s_addr = key;
+		GHashTable *id_table = value;
+		unsigned int len, r_len, s_len;
+		unsigned int i;
+		struct id_table_node *node;
+
+		if (r_addr[0] == '+' && s_addr[0] == '+')
+			continue;
+
+		if (r_addr[0] != '+' && s_addr[0] != '+')
+			continue;
+
+		r_len = strlen(r_addr);
+		s_len = strlen(s_addr);
+
+		len = MIN(6, MIN(r_len, s_len));
+
+		for (i = 0; i < len; i++)
+			if (s_addr[s_len - i - 1] != r_addr[r_len - i - 1])
+				break;
+
+		/* Not all digits matched. */
+		if (i < len)
+			continue;
+
+		/* Address matched. Check message reference. */
+		node = find_by_mr_and_mark(id_table, sr->status_report.mr,
+						out_iter, out_msgid);
+		if (node != NULL) {
+			*out_addr = s_addr;
+			return node;
+		}
+	}
+
+	return NULL;
+}
+
 gboolean status_report_assembly_report(struct status_report_assembly *assembly,
-					const struct sms *status_report,
-					unsigned int *out_id,
+					const struct sms *sr,
+					unsigned char *out_msgid,
 					gboolean *out_delivered)
 {
-	unsigned int offset = status_report->status_report.mr / 32;
-	unsigned int bit = 1 << (status_report->status_report.mr % 32);
-	struct id_table_node *node = NULL;
 	const char *straddr;
 	GHashTable *id_table;
-	gpointer key, value;
-	gboolean delivered;
 	GHashTableIter iter;
+	struct sms_address addr;
+	struct id_table_node *node;
+	gboolean delivered;
 	gboolean pending;
+	unsigned char *msgid;
 	int i;
 
 	/* We ignore temporary or tempfinal status reports */
-	if (sr_st_to_delivered(status_report->status_report.st,
-				&delivered) == FALSE)
+	if (sr_st_to_delivered(sr->status_report.st, &delivered) == FALSE)
 		return FALSE;
 
-	straddr = sms_address_to_string(&status_report->status_report.raddr);
+	straddr = sms_address_to_string(&sr->status_report.raddr);
 	id_table = g_hash_table_lookup(assembly->assembly_table, straddr);
 
-	/* key (receiver address) does not exist in assembly */
-	if (id_table == NULL)
-		return FALSE;
-
-	g_hash_table_iter_init(&iter, id_table);
-	while (g_hash_table_iter_next(&iter, &key, &value)) {
-		node = value;
-
-		if (node->mrs[offset] & bit)
-			break;
-
-		node = NULL;
-	}
+	if (id_table != NULL)
+		node = find_by_mr_and_mark(id_table, sr->status_report.mr,
+						&iter, &msgid);
+	else
+		node = fuzzy_lookup(assembly, sr, &straddr, &iter, &msgid);
 
 	/* Unable to find a message reference belonging to this address */
 	if (node == NULL)
 		return FALSE;
 
-	/* Mr belongs to this node. */
-	node->mrs[offset] ^= bit;
-
 	node->deliverable = node->deliverable && delivered;
 
 	/* If we haven't sent the entire message yet, wait until sent */
@@ -2744,27 +3021,38 @@ gboolean status_report_assembly_report(struct status_report_assembly *assembly,
 		}
 	}
 
-	if (pending == TRUE && node->deliverable == TRUE)
+	sms_address_from_string(&addr, straddr);
+
+	if (pending == TRUE && node->deliverable == TRUE) {
+		/*
+		 * More status reports expected, and already received
+		 * reports completed. Update backup file.
+		 */
+		sr_assembly_add_fragment_backup(assembly->imsi, node,
+						&addr, msgid);
+
 		return FALSE;
+	}
 
 	if (out_delivered)
 		*out_delivered = node->deliverable;
 
-	if (out_id)
-		*out_id = *((unsigned int *) key);
+	if (out_msgid)
+		memcpy(out_msgid, msgid, SMS_MSGID_LEN);
 
+	sr_assembly_remove_fragment_backup(assembly->imsi, &addr, msgid);
+	id_table = g_hash_table_iter_get_hash_table(&iter);
 	g_hash_table_iter_remove(&iter);
 
 	if (g_hash_table_size(id_table) == 0)
-		g_hash_table_remove(assembly->assembly_table,
-				status_report->status_report.raddr.address);
+		g_hash_table_remove(assembly->assembly_table, straddr);
 
 	return TRUE;
 }
 
 void status_report_assembly_add_fragment(
 					struct status_report_assembly *assembly,
-					unsigned int msg_id,
+					const unsigned char *msgid,
 					const struct sms_address *to,
 					unsigned char mr, time_t expiration,
 					unsigned char total_mrs)
@@ -2773,32 +3061,30 @@ void status_report_assembly_add_fragment(
 	unsigned int bit = 1 << (mr % 32);
 	GHashTable *id_table;
 	struct id_table_node *node;
-	unsigned int *id_table_key;
+	unsigned char *id_table_key;
 
 	id_table = g_hash_table_lookup(assembly->assembly_table,
 					sms_address_to_string(to));
 
 	/* Create hashtable keyed by the to address if required */
 	if (id_table == NULL) {
-		id_table = g_hash_table_new_full(g_int_hash, g_int_equal,
+		id_table = g_hash_table_new_full(sha1_hash, sha1_equal,
 								g_free, g_free);
 		g_hash_table_insert(assembly->assembly_table,
 					g_strdup(sms_address_to_string(to)),
 					id_table);
 	}
 
-	node = g_hash_table_lookup(id_table, &msg_id);
+	node = g_hash_table_lookup(id_table, msgid);
 
 	/* Create node in the message id hashtable if required */
 	if (node == NULL) {
-		id_table_key = g_new0(unsigned int, 1);
+		id_table_key = g_memdup(msgid, SMS_MSGID_LEN);
 
 		node = g_new0(struct id_table_node, 1);
-		memcpy(&node->to, to, sizeof(*to));
 		node->total_mrs = total_mrs;
 		node->deliverable = TRUE;
 
-		*id_table_key = msg_id;
 		g_hash_table_insert(id_table, id_table_key, node);
 	}
 
@@ -2806,13 +3092,55 @@ void status_report_assembly_add_fragment(
 	node->mrs[offset] |= bit;
 	node->expiration = expiration;
 	node->sent_mrs++;
+	sr_assembly_add_fragment_backup(assembly->imsi, node, to, msgid);
 }
 
 void status_report_assembly_expire(struct status_report_assembly *assembly,
-					time_t before, GFunc foreach_func,
-					gpointer data)
+					time_t before)
 {
-	/*TODO*/
+	GHashTable *id_table;
+	GHashTableIter iter_addr, iter_node;
+	struct sms_address addr;
+	char *straddr;
+	gpointer key;
+	struct id_table_node *node;
+
+	g_hash_table_iter_init(&iter_addr, assembly->assembly_table);
+
+	/*
+	 * Go through different addresses. Each address can relate to
+	 * 1-n msg_ids.
+	 */
+	while (g_hash_table_iter_next(&iter_addr, (gpointer) &straddr,
+					(gpointer) &id_table)) {
+
+		sms_address_from_string(&addr, straddr);
+		g_hash_table_iter_init(&iter_node, id_table);
+
+		/* Go through different messages. */
+		while (g_hash_table_iter_next(&iter_node, &key,
+						(gpointer) &node)) {
+			/*
+			 * If message is expired, removed it from the
+			 * hash-table and remove the backup-file
+			 */
+			if (node->expiration <= before) {
+				g_hash_table_iter_remove(&iter_node);
+
+				sr_assembly_remove_fragment_backup(
+								assembly->imsi,
+								&addr,
+								key);
+			}
+		}
+
+		/*
+		 * If all messages are removed, remove address
+		 * from the hash-table.
+		 */
+		if (g_hash_table_size(id_table) == 0)
+			g_hash_table_iter_remove(&iter_addr);
+	}
 }
 
 static inline GSList *sms_list_append(GSList *l, const struct sms *in)
@@ -2826,14 +3154,143 @@ static inline GSList *sms_list_append(GSList *l, const struct sms *in)
 	return l;
 }
 
-/* Prepares the text for transmission.  Breaks up into fragments if
+/*
+ * Prepares a datagram for transmission.  Breaks up into fragments if
  * necessary using ref as the concatenated message reference number.
- * Returns a list of sms messages in order.  If ref_offset is given,
- * then the ref_offset contains the reference number offset or 0
- * if no concatenation took place.
+ * Returns a list of sms messages in order.
+ *
+ * @use_delivery_reports: value for the Status-Report-Request field
+ *     (23.040 3.2.9, 9.2.2.2)
  */
-GSList *sms_text_prepare(const char *utf8, guint16 ref,
-				gboolean use_16bit, int *ref_offset,
+GSList *sms_datagram_prepare(const char *to,
+				const unsigned char *data, unsigned int len,
+				guint16 ref, gboolean use_16bit_ref,
+				unsigned short src, unsigned short dst,
+				gboolean use_16bit_port,
+				gboolean use_delivery_reports)
+{
+	struct sms template;
+	unsigned int offset;
+	unsigned int written;
+	unsigned int left;
+	guint8 seq;
+	GSList *r = NULL;
+
+	memset(&template, 0, sizeof(struct sms));
+	template.type = SMS_TYPE_SUBMIT;
+	template.submit.rd = FALSE;
+	template.submit.vpf = SMS_VALIDITY_PERIOD_FORMAT_RELATIVE;
+	template.submit.rp = FALSE;
+	template.submit.srr = use_delivery_reports;
+	template.submit.mr = 0;
+	template.submit.vp.relative = 0xA7; /* 24 Hours */
+	template.submit.dcs = 0x04; /* Class Unspecified, 8 Bit */
+	template.submit.udhi = TRUE;
+	sms_address_from_string(&template.submit.daddr, to);
+
+	offset = 1;
+
+	if (use_16bit_port) {
+		template.submit.ud[0] += 6;
+		template.submit.ud[offset] = SMS_IEI_APPLICATION_ADDRESS_16BIT;
+		template.submit.ud[offset + 1] = 4;
+		template.submit.ud[offset + 2] = (dst & 0xff00) >> 8;
+		template.submit.ud[offset + 3] = dst & 0xff;
+		template.submit.ud[offset + 4] = (src & 0xff00) >> 8;
+		template.submit.ud[offset + 5] = src & 0xff;
+
+		offset += 6;
+	} else {
+		template.submit.ud[0] += 4;
+		template.submit.ud[offset] = SMS_IEI_APPLICATION_ADDRESS_8BIT;
+		template.submit.ud[offset + 1] = 2;
+		template.submit.ud[offset + 2] = dst & 0xff;
+		template.submit.ud[offset + 3] = src & 0xff;
+
+		offset += 4;
+	}
+
+	if (len <= (140 - offset)) {
+		template.submit.udl = len + offset;
+		memcpy(template.submit.ud + offset, data, len);
+
+		return sms_list_append(NULL, &template);
+	}
+
+	if (use_16bit_ref) {
+		template.submit.ud[0] += 6;
+		template.submit.ud[offset] = SMS_IEI_CONCATENATED_16BIT;
+		template.submit.ud[offset + 1] = 4;
+		template.submit.ud[offset + 2] = (ref & 0xff00) >> 8;
+		template.submit.ud[offset + 3] = ref & 0xff;
+
+		offset += 6;
+	} else {
+		template.submit.ud[0] += 5;
+		template.submit.ud[offset] = SMS_IEI_CONCATENATED_8BIT;
+		template.submit.ud[offset + 1] = 3;
+		template.submit.ud[offset + 2] = ref & 0xff;
+
+		offset += 5;
+	}
+
+	seq = 0;
+	left = len;
+	written = 0;
+
+	while (left > 0) {
+		unsigned int chunk;
+
+		seq += 1;
+
+		chunk = 140 - offset;
+		if (left < chunk)
+			chunk = left;
+
+		template.submit.udl = chunk + offset;
+		memcpy(template.submit.ud + offset, data + written, chunk);
+
+		written += chunk;
+		left -= chunk;
+
+		template.submit.ud[offset - 1] = seq;
+
+		r = sms_list_append(r, &template);
+
+		if (seq == 255)
+			break;
+	}
+
+	if (left > 0) {
+		g_slist_foreach(r, (GFunc) g_free, NULL);
+		g_slist_free(r);
+
+		return NULL;
+	} else {
+		GSList *l;
+
+		for (l = r; l; l = l->next) {
+			struct sms *sms = l->data;
+
+			sms->submit.ud[offset - 2] = seq;
+		}
+	}
+
+	r = g_slist_reverse(r);
+
+	return r;
+}
+
+/*
+ * Prepares the text for transmission.  Breaks up into fragments if
+ * necessary using ref as the concatenated message reference number.
+ * Returns a list of sms messages in order.
+ *
+ * @use_delivery_reports: value for the Status-Report-Request field
+ *     (23.040 3.2.9, 9.2.2.2)
+ */
+GSList *sms_text_prepare(const char *to, const char *utf8, guint16 ref,
+				gboolean use_16bit,
 				gboolean use_delivery_reports)
 {
 	struct sms template;
@@ -2853,11 +3310,11 @@ GSList *sms_text_prepare(const char *utf8, guint16 ref,
 	template.submit.srr = use_delivery_reports;
 	template.submit.mr = 0;
 	template.submit.vp.relative = 0xA7; /* 24 Hours */
+	sms_address_from_string(&template.submit.daddr, to);
 
 	/* UDHI, UDL, UD and DCS actually depend on what we have in the text */
 	gsm_encoded = convert_utf8_to_gsm(utf8, -1, NULL, &written, 0);
-
-	if (!gsm_encoded) {
+	if (gsm_encoded == NULL) {
 		gsize converted;
 
 		ucs2_encoded = g_convert(utf8, -1, "UCS-2BE//TRANSLIT", "UTF-8",
@@ -2865,7 +3322,7 @@ GSList *sms_text_prepare(const char *utf8, guint16 ref,
 		written = converted;
 	}
 
-	if (!gsm_encoded && !ucs2_encoded)
+	if (gsm_encoded == NULL && ucs2_encoded == NULL)
 		return NULL;
 
 	if (gsm_encoded)
@@ -2877,9 +3334,6 @@ GSList *sms_text_prepare(const char *utf8, guint16 ref,
 		template.submit.udhi = FALSE;
 
 	if (gsm_encoded && (written <= sms_text_capacity_gsm(160, offset))) {
-		if (ref_offset)
-			*ref_offset = 0;
-
 		template.submit.udl = written + (offset * 8 + 6) / 7;
 		pack_7bit_own_buf(gsm_encoded, written, offset, FALSE, NULL,
 					0, template.submit.ud + offset);
@@ -2889,9 +3343,6 @@ GSList *sms_text_prepare(const char *utf8, guint16 ref,
 	}
 
 	if (ucs2_encoded && (written <= (140 - offset))) {
-		if (ref_offset)
-			*ref_offset = 0;
-
 		template.submit.udl = written + offset;
 		memcpy(template.submit.ud + offset, ucs2_encoded, written);
 
@@ -2904,22 +3355,19 @@ GSList *sms_text_prepare(const char *utf8, guint16 ref,
 	if (!offset)
 		offset = 1;
 
-	if (ref_offset)
-		*ref_offset = offset + 2;
-
 	if (use_16bit) {
 		template.submit.ud[0] += 6;
 		template.submit.ud[offset] = SMS_IEI_CONCATENATED_16BIT;
 		template.submit.ud[offset + 1] = 4;
-		template.submit.ud[offset + 2] = (ref & 0xf0) >> 8;
-		template.submit.ud[offset + 3] = ref & 0xf;
+		template.submit.ud[offset + 2] = (ref & 0xff00) >> 8;
+		template.submit.ud[offset + 3] = ref & 0xff;
 
 		offset += 6;
 	} else {
 		template.submit.ud[0] += 5;
 		template.submit.ud[offset] = SMS_IEI_CONCATENATED_8BIT;
 		template.submit.ud[offset + 1] = 3;
-		template.submit.ud[offset + 2] = ref & 0xf;
+		template.submit.ud[offset + 2] = ref & 0xff;
 
 		offset += 5;
 	}
@@ -2976,7 +3424,7 @@ GSList *sms_text_prepare(const char *utf8, guint16 ref,
 		g_free(ucs2_encoded);
 
 	if (left > 0) {
-		g_slist_foreach(r, (GFunc)g_free, NULL);
+		g_slist_foreach(r, (GFunc) g_free, NULL);
 		g_slist_free(r);
 
 		return NULL;
@@ -3015,7 +3463,7 @@ gboolean cbs_dcs_decode(guint8 dcs, gboolean *udhi, enum sms_class *cls,
 	case 0:
 		ch = SMS_CHARSET_7BIT;
 		cl = SMS_CLASS_UNSPECIFIED;
-		lang = (enum cbs_language)lower;
+		lang = (enum cbs_language) lower;
 		break;
 	case 1:
 		if (lower > 1)
@@ -3036,7 +3484,7 @@ gboolean cbs_dcs_decode(guint8 dcs, gboolean *udhi, enum sms_class *cls,
 
 		ch = SMS_CHARSET_7BIT;
 		cl = SMS_CLASS_UNSPECIFIED;
-		lang = (enum cbs_language)dcs;
+		lang = (enum cbs_language) dcs;
 		break;
 	case 4:
 	case 5:
@@ -3045,21 +3493,21 @@ gboolean cbs_dcs_decode(guint8 dcs, gboolean *udhi, enum sms_class *cls,
 		comp = (dcs & 0x20) ? TRUE : FALSE;
 
 		if (dcs & 0x10)
-			cl = (enum sms_class)(dcs & 0x03);
+			cl = (enum sms_class) (dcs & 0x03);
 		else
 			cl = SMS_CLASS_UNSPECIFIED;
 
 		if (((dcs & 0x0c) >> 2) < 3)
-			ch = (enum sms_charset)((dcs & 0x0c) >> 2);
+			ch = (enum sms_charset) ((dcs & 0x0c) >> 2);
 		else
 			return FALSE;
 
 		break;
 	case 9:
 		udh = TRUE;
-		cl = (enum sms_class)(dcs & 0x03);
+		cl = (enum sms_class) (dcs & 0x03);
 		if (((dcs & 0x0c) >> 2) < 3)
-			ch = (enum sms_charset)((dcs & 0x0c) >> 2);
+			ch = (enum sms_charset) ((dcs & 0x0c) >> 2);
 		else
 			return FALSE;
 
@@ -3074,7 +3522,7 @@ gboolean cbs_dcs_decode(guint8 dcs, gboolean *udhi, enum sms_class *cls,
 			ch = SMS_CHARSET_7BIT;
 
 		if (lower & 0x3)
-			cl = (enum sms_class)(lower & 0x3);
+			cl = (enum sms_class) (lower & 0x3);
 		else
 			cl = SMS_CLASS_UNSPECIFIED;
 
@@ -3118,7 +3566,8 @@ gboolean cbs_decode(const unsigned char *pdu, int len, struct cbs *out)
 	out->max_pages = pdu[5] & 0xf;
 	out->page = (pdu[5] >> 4) & 0xf;
 
-	/* If a mobile receives the code 0000 in either the first field or
+	/*
+	 * If a mobile receives the code 0000 in either the first field or
 	 * the second field then it shall treat the CBS message exactly the
 	 * same as a CBS message with page parameter 0001 0001 (i.e. a single
 	 * page message).
@@ -3286,7 +3735,8 @@ char *cbs_decode_text(GSList *cbs_list, char *iso639_lang)
 	if (cbs_list == NULL)
 		return NULL;
 
-	/* CBS can only come from the network, so we're much less lenient
+	/*
+	 * CBS can only come from the network, so we're much less lenient
 	 * on what we support.  Namely we require the same charset to be
 	 * used across all pages.
 	 */
@@ -3376,7 +3826,8 @@ char *cbs_decode_text(GSList *cbs_list, char *iso639_lang)
 
 			i = iso639 ? 3 : 0;
 
-			/* CR is a padding character, which means we can
+			/*
+			 * CR is a padding character, which means we can
 			 * safely discard everything afterwards
 			 */
 			for (; i < written; i++, bufsize++) {
@@ -3386,7 +3837,8 @@ char *cbs_decode_text(GSList *cbs_list, char *iso639_lang)
 				buf[bufsize] = unpacked[i];
 			}
 
-			/* It isn't clear whether extension sequences
+			/*
+			 * It isn't clear whether extension sequences
 			 * (2 septets) must be wholly present in the page
 			 * and not broken over multiple pages.  The behavior
 			 * is probably the same as SMS, but we don't make
@@ -3397,9 +3849,10 @@ char *cbs_decode_text(GSList *cbs_list, char *iso639_lang)
 			int i = taken;
 			int max_offset = taken + num_ucs2_chars * 2;
 
-			/* It is completely unclear how UCS2 chars are handled
+			/*
+			 * It is completely unclear how UCS2 chars are handled
 			 * especially across pages or when the UDH is present.
-			 * For now do the best we can
+			 * For now do the best we can.
 			 */
 			if (iso639) {
 				i += 2;
@@ -3422,7 +3875,7 @@ char *cbs_decode_text(GSList *cbs_list, char *iso639_lang)
 	if (charset == SMS_CHARSET_7BIT)
 		utf8 = convert_gsm_to_utf8(buf, bufsize, NULL, NULL, 0);
 	else
-		utf8 = g_convert((char *)buf, bufsize, "UTF-8//TRANSLIT",
+		utf8 = g_convert((char *) buf, bufsize, "UTF-8//TRANSLIT",
 					"UCS-2BE", NULL, NULL, NULL);
 
 	g_free(buf);
@@ -3437,7 +3890,8 @@ static inline gboolean cbs_is_update_newer(unsigned int n, unsigned int o)
 	if (new_update == old_update)
 		return FALSE;
 
-	/* Any Update Number eight or less higher (modulo 16) than the last
+	/*
+	 * Any Update Number eight or less higher (modulo 16) than the last
 	 * received Update Number will be considered more recent, and shall be
 	 * treated as a new CBS message, provided the mobile has not been
 	 * switched off.
@@ -3460,7 +3914,7 @@ void cbs_assembly_free(struct cbs_assembly *assembly)
 	for (l = assembly->assembly_list; l; l = l->next) {
 		struct cbs_assembly_node *node = l->data;
 
-		g_slist_foreach(node->pages, (GFunc)g_free, 0);
+		g_slist_foreach(node->pages, (GFunc) g_free, 0);
 		g_slist_free(node->pages);
 		g_free(node);
 	}
@@ -3516,8 +3970,9 @@ static void cbs_assembly_expire(struct cbs_assembly *assembly,
 	GSList *prev;
 	GSList *tmp;
 
-	/* Take care of the case where several updates are being
-	 * reassembled at the same time.  If the newer one is assembled
+	/*
+	 * Take care of the case where several updates are being
+	 * reassembled at the same time. If the newer one is assembled
 	 * first, then the subsequent old update is discarded, make
 	 * sure that we're also discarding the assembly node for the
 	 * partially assembled ones
@@ -3539,7 +3994,7 @@ static void cbs_assembly_expire(struct cbs_assembly *assembly,
 		else
 			assembly->assembly_list = l->next;
 
-		g_slist_foreach(node->pages, (GFunc)g_free, NULL);
+		g_slist_foreach(node->pages, (GFunc) g_free, NULL);
 		g_slist_free(node->pages);
 		g_free(node->pages);
 		tmp = l;
@@ -3551,7 +4006,8 @@ static void cbs_assembly_expire(struct cbs_assembly *assembly,
 void cbs_assembly_location_changed(struct cbs_assembly *assembly, gboolean plmn,
 					gboolean lac, gboolean ci)
 {
-	/* Location Area wide (in GSM) (which means that a CBS message with the
+	/*
+	 * Location Area wide (in GSM) (which means that a CBS message with the
 	 * same Message Code and Update Number may or may not be "new" in the
 	 * next cell according to whether the next cell is in the same Location
 	 * Area as the current cell), or
@@ -3848,7 +4304,7 @@ GSList *cbs_extract_topic_ranges(const char *ranges)
 	}
 
 	tmp = cbs_optimize_ranges(ret);
-	g_slist_foreach(ret, (GFunc)g_free, NULL);
+	g_slist_foreach(ret, (GFunc) g_free, NULL);
 	g_slist_free(ret);
 
 	return tmp;
@@ -3931,7 +4387,7 @@ static gint cbs_topic_compare(gconstpointer a, gconstpointer b)
 
 gboolean cbs_topic_in_range(unsigned int topic, GSList *ranges)
 {
-	if (!ranges)
+	if (ranges == NULL)
 		return FALSE;
 
 	return g_slist_find_custom(ranges, GUINT_TO_POINTER(topic),
@@ -3981,3 +4437,30 @@ char *ussd_decode(int dcs, int len, const unsigned char *data)
 
 	return utf8;
 }
+
+gboolean ussd_encode(const char *str, long *items_written, unsigned char *pdu)
+{
+	unsigned char *converted = NULL;
+	long written;
+	long num_packed;
+
+	if (pdu == NULL)
+		return FALSE;
+
+	converted = convert_utf8_to_gsm(str, -1, NULL, &written, 0);
+	if (converted == NULL || written > 182) {
+		g_free(converted);
+		return FALSE;
+	}
+
+	pack_7bit_own_buf(converted, written, 0, TRUE, &num_packed, 0, pdu);
+	g_free(converted);
+
+	if (num_packed < 1)
+		return FALSE;
+
+	if (items_written)
+		*items_written = num_packed;
+
+	return TRUE;
+}
