@@ -68,6 +68,7 @@ struct voicecall {
 	char *message;
 	uint8_t icon_id;
 	gboolean untracked;
+	gboolean dial_result_handled;
 };
 
 struct dial_request {
@@ -177,6 +178,17 @@ static const char *phone_and_clip_to_string(const struct ofono_phone_number *n,
 	return phone_number_to_string(n);
 }
 
+static const char *cnap_to_string(const char *name, int cnap_validity)
+{
+	if (cnap_validity == CNAP_VALIDITY_WITHHELD && !strlen(name))
+		return "withheld";
+
+	if (cnap_validity == CNAP_VALIDITY_NOT_AVAILABLE)
+		return "";
+
+	return name;
+}
+
 static const char *time_to_str(const time_t *t)
 {
 	static char buf[128];
@@ -315,6 +327,23 @@ static void tone_request_finish(struct ofono_voicecall *vc,
 	g_free(entry);
 }
 
+static gint number_compare(gconstpointer a, gconstpointer b)
+{
+	const char *s1 = a, *s2 = b;
+	return strcmp(s1, s2);
+}
+
+static gboolean voicecall_is_emergency(struct voicecall *v)
+{
+	struct ofono_call *call = v->call;
+	const char *lineid_str;
+
+	lineid_str = phone_number_to_string(&call->phone_number);
+
+	return g_slist_find_custom(v->vc->en_list, lineid_str,
+						number_compare) ? TRUE : FALSE;
+}
+
 static void append_voicecall_properties(struct voicecall *v,
 					DBusMessageIter *dict)
 {
@@ -322,16 +351,27 @@ static void append_voicecall_properties(struct voicecall *v,
 	const char *status;
 	const char *callerid;
 	const char *timestr;
+	const char *name;
 	ofono_bool_t mpty;
+	dbus_bool_t emergency_call;
 
 	status = call_status_to_string(call->status);
-	callerid = phone_number_to_string(&call->phone_number);
+
+	if (call->direction == CALL_DIRECTION_MOBILE_TERMINATED)
+		callerid = phone_and_clip_to_string(&call->phone_number,
+							call->clip_validity);
+	else
+		callerid = phone_number_to_string(&call->phone_number);
+
+	name = cnap_to_string(call->name, call->cnap_validity);
 
 	ofono_dbus_dict_append(dict, "State", DBUS_TYPE_STRING, &status);
 
 	ofono_dbus_dict_append(dict, "LineIdentification",
 				DBUS_TYPE_STRING, &callerid);
 
+	ofono_dbus_dict_append(dict, "Name", DBUS_TYPE_STRING, &name);
+
 	if (call->status == CALL_STATUS_ACTIVE ||
 			call->status == CALL_STATUS_HELD ||
 			(call->status == CALL_STATUS_DISCONNECTED &&
@@ -358,6 +398,15 @@ static void append_voicecall_properties(struct voicecall *v,
 	if (v->icon_id)
 		ofono_dbus_dict_append(dict, "Icon", DBUS_TYPE_BYTE,
 					&v->icon_id);
+
+	if (voicecall_is_emergency(v) == TRUE)
+		emergency_call = TRUE;
+	else
+		emergency_call = FALSE;
+
+	ofono_dbus_dict_append(dict, "Emergency", DBUS_TYPE_BOOLEAN,
+					&emergency_call);
+
 }
 
 static DBusMessage *voicecall_get_properties(DBusConnection *conn,
@@ -369,8 +418,7 @@ static DBusMessage *voicecall_get_properties(DBusConnection *conn,
 	DBusMessageIter dict;
 
 	reply = dbus_message_new_method_return(msg);
-
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -398,7 +446,7 @@ static DBusMessage *voicecall_deflect(DBusConnection *conn,
 		call->status != CALL_STATUS_WAITING)
 		return __ofono_error_failed(msg);
 
-	if (!vc->driver->deflect)
+	if (vc->driver->deflect == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (vc->pending)
@@ -423,10 +471,10 @@ static DBusMessage *voicecall_deflect(DBusConnection *conn,
 static void dial_request_user_cancel(struct ofono_voicecall *vc,
 					struct voicecall *call)
 {
-	if (!vc->dial_req)
+	if (vc->dial_req == NULL)
 		return;
 
-	if (!call || call == vc->dial_req->call)
+	if (call == NULL || call == vc->dial_req->call)
 		dial_request_finish(vc);
 }
 
@@ -533,7 +581,7 @@ static DBusMessage *voicecall_answer(DBusConnection *conn,
 	if (call->status != CALL_STATUS_INCOMING)
 		return __ofono_error_failed(msg);
 
-	if (!vc->driver->answer)
+	if (vc->driver->answer == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (vc->pending)
@@ -569,8 +617,7 @@ static struct voicecall *voicecall_create(struct ofono_voicecall *vc,
 	struct voicecall *v;
 
 	v = g_try_new0(struct voicecall, 1);
-
-	if (!v)
+	if (v == NULL)
 		return NULL;
 
 	v->call = call;
@@ -691,8 +738,8 @@ static void voicecall_set_call_lineid(struct voicecall *v,
 	const char *lineid_str;
 
 	if (!strcmp(call->phone_number.number, ph->number) &&
-		call->phone_number.type == ph->type &&
-		call->clip_validity == clip_validity)
+			call->phone_number.type == ph->type &&
+			call->clip_validity == clip_validity)
 		return;
 
 	/*
@@ -704,7 +751,7 @@ static void voicecall_set_call_lineid(struct voicecall *v,
 
 	/* For plugins that don't keep state, ignore */
 	if (call->clip_validity == CLIP_VALIDITY_VALID &&
-		clip_validity == CLIP_VALIDITY_NOT_AVAILABLE)
+			clip_validity == CLIP_VALIDITY_NOT_AVAILABLE)
 		return;
 
 	strcpy(call->phone_number.number, ph->number);
@@ -722,6 +769,47 @@ static void voicecall_set_call_lineid(struct voicecall *v,
 						OFONO_VOICECALL_INTERFACE,
 						"LineIdentification",
 						DBUS_TYPE_STRING, &lineid_str);
+
+	if (voicecall_is_emergency(v)) {
+		dbus_bool_t emergency_call = TRUE;
+
+		ofono_dbus_signal_property_changed(conn, path,
+						OFONO_VOICECALL_INTERFACE,
+						"Emergency",
+						DBUS_TYPE_BOOLEAN,
+						&emergency_call);
+	}
+}
+
+static void voicecall_set_call_name(struct voicecall *v,
+					const char *name,
+					int cnap_validity)
+{
+	struct ofono_call *call = v->call;
+	DBusConnection *conn = ofono_dbus_get_connection();
+	const char *path;
+	const char *name_str;
+
+	if (!strcmp(call->name, name) && call->cnap_validity == cnap_validity)
+		return;
+
+	/* For plugins that don't keep state, ignore */
+	if (call->cnap_validity == CNAP_VALIDITY_VALID &&
+			cnap_validity == CNAP_VALIDITY_NOT_AVAILABLE)
+		return;
+
+	strncpy(call->name, name, OFONO_MAX_CALLER_NAME_LENGTH);
+	call->name[OFONO_MAX_CALLER_NAME_LENGTH] = '\0';
+	call->cnap_validity = cnap_validity;
+
+	path = voicecall_build_path(v->vc, call);
+
+	name_str = cnap_to_string(name, cnap_validity);
+
+	ofono_dbus_signal_property_changed(conn, path,
+						OFONO_VOICECALL_INTERFACE,
+						"Name",
+						DBUS_TYPE_STRING, &name_str);
 }
 
 static gboolean voicecall_dbus_register(struct voicecall *v)
@@ -729,7 +817,7 @@ static gboolean voicecall_dbus_register(struct voicecall *v)
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *path;
 
-	if (!v)
+	if (v == NULL)
 		return FALSE;
 
 	path = voicecall_build_path(v->vc, v->call);
@@ -879,10 +967,10 @@ static void voicecalls_multiparty_changed(GSList *old, GSList *new)
 		nc = n ? n->data : NULL;
 		oc = o ? o->data : NULL;
 
-		if (oc && (!nc || (nc->call->id > oc->call->id))) {
+		if (oc && (nc == NULL || (nc->call->id > oc->call->id))) {
 			voicecall_emit_multiparty(oc, FALSE);
 			o = o->next;
-		} else if (nc && (!oc || (nc->call->id < oc->call->id))) {
+		} else if (nc && (oc == NULL || (nc->call->id < oc->call->id))) {
 			voicecall_emit_multiparty(nc, TRUE);
 			n = n->next;
 		} else {
@@ -957,7 +1045,7 @@ static void voicecalls_release_next(struct ofono_voicecall *vc)
 {
 	struct voicecall *call;
 
-	if (!vc->release_list)
+	if (vc->release_list == NULL)
 		return;
 
 	call = vc->release_list->data;
@@ -997,8 +1085,7 @@ static DBusMessage *manager_get_properties(DBusConnection *conn,
 	char **list;
 
 	reply = dbus_message_new_method_return(msg);
-
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -1046,8 +1133,7 @@ static struct ofono_call *synthesize_outgoing_call(struct ofono_voicecall *vc,
 	struct ofono_call *call;
 
 	call = g_try_new0(struct ofono_call, 1);
-
-	if (!call)
+	if (call == NULL)
 		return call;
 
 	call->id = __ofono_modem_callid_next(modem);
@@ -1096,17 +1182,26 @@ static struct voicecall *dial_handle_result(struct ofono_voicecall *vc,
 		v = l->data;
 
 		if (v->call->status == CALL_STATUS_DIALING ||
-				v->call->status == CALL_STATUS_ALERTING ||
-				v->call->status == CALL_STATUS_ACTIVE)
-			return v;
+				v->call->status == CALL_STATUS_ALERTING)
+			goto handled;
+
+		/*
+		 * Dial request may return before existing active call
+		 * is put on hold or after dialed call has got active
+		 */
+		if (v->call->status == CALL_STATUS_ACTIVE &&
+				v->call->direction ==
+				CALL_DIRECTION_MOBILE_ORIGINATED &&
+				!v->dial_result_handled)
+			goto handled;
 	}
 
 	call = synthesize_outgoing_call(vc, number);
-	if (!call)
+	if (call == NULL)
 		return NULL;
 
 	v = voicecall_create(vc, call);
-	if (!v)
+	if (v == NULL)
 		return NULL;
 
 	v->detect_time = time(NULL);
@@ -1119,6 +1214,9 @@ static struct voicecall *dial_handle_result(struct ofono_voicecall *vc,
 
 	*need_to_emit = TRUE;
 
+handled:
+	v->dial_result_handled = TRUE;
+
 	return v;
 }
 
@@ -1178,7 +1276,7 @@ static DBusMessage *manager_dial(DBusConnection *conn,
 	if (clir_string_to_clir(clirstr, &clir) == FALSE)
 		return __ofono_error_invalid_format(msg);
 
-	if (!vc->driver->dial)
+	if (vc->driver->dial == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (voicecalls_have_incoming(vc))
@@ -1225,7 +1323,7 @@ static DBusMessage *manager_transfer(DBusConnection *conn,
 	if ((numactive != 1) && (numheld != 1))
 		return __ofono_error_failed(msg);
 
-	if (!vc->driver->transfer)
+	if (vc->driver->transfer == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	vc->pending = dbus_message_ref(msg);
@@ -1265,7 +1363,7 @@ static DBusMessage *manager_swap_calls(DBusConnection *conn,
 	if (voicecalls_have_waiting(vc))
 		return __ofono_error_failed(msg);
 
-	if (!vc->driver->hold_all_active)
+	if (vc->driver->hold_all_active == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	vc->pending = dbus_message_ref(msg);
@@ -1286,7 +1384,7 @@ static DBusMessage *manager_release_and_answer(DBusConnection *conn,
 	if (!voicecalls_have_waiting(vc))
 		return __ofono_error_failed(msg);
 
-	if (!vc->driver->release_all_active)
+	if (vc->driver->release_all_active == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	vc->pending = dbus_message_ref(msg);
@@ -1314,7 +1412,7 @@ static DBusMessage *manager_hold_and_answer(DBusConnection *conn,
 	if (voicecalls_have_active(vc) && voicecalls_have_held(vc))
 		return __ofono_error_failed(msg);
 
-	if (!vc->driver->hold_all_active)
+	if (vc->driver->hold_all_active == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	vc->pending = dbus_message_ref(msg);
@@ -1448,7 +1546,7 @@ static DBusMessage *multiparty_private_chat(DBusConnection *conn,
 
 	c = strrchr(callpath, '/');
 
-	if (!c || strncmp(path, callpath, c-callpath))
+	if (c == NULL || strncmp(path, callpath, c-callpath))
 		return __ofono_error_not_found(msg);
 
 	if (!sscanf(c, "/voicecall%2u", &id))
@@ -1460,7 +1558,7 @@ static DBusMessage *multiparty_private_chat(DBusConnection *conn,
 			break;
 	}
 
-	if (!l)
+	if (l == NULL)
 		return __ofono_error_not_found(msg);
 
 	/*
@@ -1471,7 +1569,7 @@ static DBusMessage *multiparty_private_chat(DBusConnection *conn,
 	if (voicecalls_have_held(vc))
 		return __ofono_error_failed(msg);
 
-	if (!vc->driver->private_chat)
+	if (vc->driver->private_chat == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	vc->pending = dbus_message_ref(msg);
@@ -1540,7 +1638,7 @@ static DBusMessage *multiparty_create(DBusConnection *conn,
 	if (!voicecalls_have_held(vc) || !voicecalls_have_active(vc))
 		return __ofono_error_failed(msg);
 
-	if (!vc->driver->create_multiparty)
+	if (vc->driver->create_multiparty == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	vc->pending = dbus_message_ref(msg);
@@ -1558,13 +1656,13 @@ static DBusMessage *multiparty_hangup(DBusConnection *conn,
 	if (vc->pending)
 		return __ofono_error_busy(msg);
 
-	if (!vc->driver->release_specific)
+	if (vc->driver->release_specific == NULL)
 		return __ofono_error_not_implemented(msg);
 
-	if (!vc->driver->release_all_held)
+	if (vc->driver->release_all_held == NULL)
 		return __ofono_error_not_implemented(msg);
 
-	if (!vc->driver->release_all_active)
+	if (vc->driver->release_all_active == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (vc->multiparty_list == NULL) {
@@ -1628,7 +1726,7 @@ static DBusMessage *manager_tone(DBusConnection *conn,
 	if (vc->pending)
 		return __ofono_error_busy(msg);
 
-	if (!vc->driver->send_tones)
+	if (vc->driver->send_tones == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	/* Send DTMFs only if we have at least one connected call */
@@ -1760,7 +1858,7 @@ void ofono_voicecall_disconnected(struct ofono_voicecall *vc, int id,
 	l = g_slist_find_custom(vc->call_list, GUINT_TO_POINTER(id),
 				call_compare_by_id);
 
-	if (!l) {
+	if (l == NULL) {
 		ofono_error("Plugin notified us of call disconnect for"
 				" unknown call");
 		return;
@@ -1825,28 +1923,28 @@ void ofono_voicecall_notify(struct ofono_voicecall *vc,
 				call_compare_by_id);
 
 	if (l) {
-		DBG("Found call with id: %d\n", call->id);
+		DBG("Found call with id: %d", call->id);
 		voicecall_set_call_status(l->data, call->status);
 		voicecall_set_call_lineid(l->data, &call->phone_number,
 						call->clip_validity);
+		voicecall_set_call_name(l->data, call->name,
+						call->cnap_validity);
 
 		return;
 	}
 
-	DBG("Did not find a call with id: %d\n", call->id);
+	DBG("Did not find a call with id: %d", call->id);
 
 	__ofono_modem_callid_hold(modem, call->id);
 
 	newcall = g_memdup(call, sizeof(struct ofono_call));
-
-	if (!newcall) {
+	if (newcall == NULL) {
 		ofono_error("Unable to allocate call");
 		goto error;
 	}
 
 	v = voicecall_create(vc, newcall);
-
-	if (!v) {
+	if (v == NULL) {
 		ofono_error("Unable to allocate voicecall_data");
 		goto error;
 	}
@@ -2363,7 +2461,7 @@ int __ofono_voicecall_dial(struct ofono_voicecall *vc,
 	if (!valid_phone_number_format(addr))
 		return -EINVAL;
 
-	if (!vc->driver->dial)
+	if (vc->driver->dial == NULL)
 		return -ENOSYS;
 
 	if (interaction == OFONO_VOICECALL_INTERACTION_DISCONNECT &&
@@ -2419,7 +2517,7 @@ int __ofono_voicecall_dial(struct ofono_voicecall *vc,
 
 void __ofono_voicecall_dial_cancel(struct ofono_voicecall *vc)
 {
-	if (!vc->dial_req || !vc->dial_req->cb)
+	if (vc->dial_req == NULL || vc->dial_req->cb == NULL)
 		return;
 
 	vc->dial_req->cb = NULL;
@@ -2431,7 +2529,7 @@ static void tone_request_cb(const struct ofono_error *error, void *data)
 	struct tone_queue_entry *entry = g_queue_peek_head(vc->toneq);
 	int len = 0;
 
-	if (!entry)
+	if (entry == NULL)
 		return;
 
 	/*
@@ -2477,7 +2575,7 @@ static gboolean tone_request_run(gpointer user_data)
 
 	vc->tone_source = 0;
 
-	if (!entry)
+	if (entry == NULL)
 		return FALSE;
 
 	len = strcspn(entry->left, "pP");
@@ -2504,7 +2602,7 @@ int __ofono_voicecall_tone_send(struct ofono_voicecall *vc,
 				const char *tone_str,
 				ofono_voicecall_tone_cb_t cb, void *user_data)
 {
-	if (!vc->driver->send_tones)
+	if (vc->driver->send_tones == NULL)
 		return -ENOSYS;
 
 	/* Send DTMFs only if we have at least one connected call */
